@@ -115,6 +115,7 @@ new_widget_info (const char *name,
   else
     {
       info->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+      gtk_window_set_has_resize_grip (GTK_WINDOW (info->window), FALSE);
       info->include_decorations = FALSE;
       gtk_widget_show_all (widget);
       gtk_container_add (GTK_CONTAINER (info->window), widget);
@@ -159,6 +160,26 @@ create_button (void)
   return new_widget_info ("button", align, SMALL);
 }
 
+static WidgetInfo *
+create_switch (void)
+{
+  GtkWidget *widget;
+  GtkWidget *align;
+  GtkWidget *sw;
+
+  widget = gtk_box_new (GTK_ORIENTATION_VERTICAL, 3);
+  sw = gtk_switch_new ();
+  gtk_switch_set_active (GTK_SWITCH (sw), TRUE);
+  gtk_box_pack_start (GTK_BOX (widget), sw, TRUE, TRUE, 0);
+  sw = gtk_switch_new ();
+  gtk_box_pack_start (GTK_BOX (widget), sw, TRUE, TRUE, 0);
+
+  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  gtk_container_add (GTK_CONTAINER (align), widget);
+
+  return new_widget_info ("switch", align, SMALL);
+}
+
 static WidgetInfo *
 create_toggle_button (void)
 {
@@ -200,6 +221,54 @@ create_link_button (void)
   return new_widget_info ("link-button", align, SMALL);
 }
 
+#define G_TYPE_TEST_PERMISSION      (g_test_permission_get_type ())
+#define G_TEST_PERMISSION(inst)     (G_TYPE_CHECK_INSTANCE_CAST ((inst), \
+                                     G_TYPE_TEST_PERMISSION,             \
+                                     GTestPermission))
+#define G_IS_TEST_PERMISSION(inst)  (G_TYPE_CHECK_INSTANCE_TYPE ((inst), \
+                                     G_TYPE_TEST_PERMISSION))
+
+typedef struct _GTestPermission GTestPermission;
+typedef struct _GTestPermissionClass GTestPermissionClass;
+
+struct _GTestPermission
+{
+  GPermission parent;
+
+  gboolean success;
+};
+
+struct _GTestPermissionClass
+{
+  GPermissionClass parent_class;
+};
+
+G_DEFINE_TYPE (GTestPermission, g_test_permission, G_TYPE_PERMISSION)
+
+static void
+g_test_permission_init (GTestPermission *test)
+{
+  g_permission_impl_update (G_PERMISSION (test), FALSE, TRUE, TRUE);
+}
+
+static void
+g_test_permission_class_init (GTestPermissionClass *class)
+{
+}
+
+static WidgetInfo *
+create_lockbutton (void)
+{
+  GtkWidget *widget;
+  GtkWidget *align;
+
+  widget = gtk_lock_button_new (g_object_new (G_TYPE_TEST_PERMISSION, NULL));
+  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  gtk_container_add (GTK_CONTAINER (align), widget);
+
+  return new_widget_info ("lock-button", align, SMALL);
+}
+
 static WidgetInfo *
 create_entry (void)
 {
@@ -628,23 +697,23 @@ create_colorsel (void)
   GtkWidget *widget;
   GtkColorSelection *colorsel;
   GtkColorSelectionDialog *selection_dialog;
-  GdkColor color;
+  GdkRGBA rgba;
 
   widget = gtk_color_selection_dialog_new ("Color Selection Dialog");
   selection_dialog = GTK_COLOR_SELECTION_DIALOG (widget);
   colorsel = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (selection_dialog));
 
-  color.red   = 0x7979;
-  color.green = 0xdbdb;
-  color.blue  = 0x9595;
+  rgba.red   = 0.4745;
+  rgba.green = 0.8588;
+  rgba.blue  = 0.5843;
 
-  gtk_color_selection_set_previous_color (colorsel, &color);
-  
-  color.red   = 0x7d7d;
-  color.green = 0x9393;
-  color.blue  = 0xc3c3;
-  
-  gtk_color_selection_set_current_color (colorsel, &color);
+  gtk_color_selection_set_previous_rgba (colorsel, &rgba);
+
+  rgba.red   = 0.4902;
+  rgba.green = 0.5764;
+  rgba.blue  = 0.7647;
+
+  gtk_color_selection_set_current_rgba (colorsel, &rgba);
 
   info = new_widget_info ("colorsel", widget, ASIS);
   info->include_decorations = TRUE;
@@ -1049,6 +1118,39 @@ create_assistant (void)
   return info;
 }
 
+static WidgetInfo *
+create_appchooserbutton (void)
+{
+  GtkWidget *picker;
+  GtkWidget *align, *vbox;
+
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 3);
+  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+  picker = gtk_app_chooser_button_new ("text/plain");
+  gtk_container_add (GTK_CONTAINER (align), picker);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox),
+                      gtk_label_new ("Application Button"),
+                      FALSE, FALSE, 0);
+
+  return new_widget_info ("appchooserbutton", vbox, SMALL);
+}
+
+static WidgetInfo *
+create_appchooserdialog (void)
+{
+  WidgetInfo *info;
+  GtkWidget *widget;
+
+  widget = gtk_app_chooser_dialog_new_for_content_type (NULL, 0, "image/png");
+  gtk_window_set_default_size (GTK_WINDOW (widget), 200, 300);
+
+  info = new_widget_info ("appchooserdialog", widget, ASIS);
+  info->include_decorations = TRUE;
+
+  return info;
+}
+
 GList *
 get_all_widgets (void)
 {
@@ -1095,6 +1197,10 @@ get_all_widgets (void)
   retval = g_list_prepend (retval, create_page_setup_dialog ());
   retval = g_list_prepend (retval, create_print_dialog ());
   retval = g_list_prepend (retval, create_volume_button ());
+  retval = g_list_prepend (retval, create_switch ());
+  retval = g_list_prepend (retval, create_appchooserbutton ());
+  retval = g_list_prepend (retval, create_appchooserdialog ());
+  retval = g_list_prepend (retval, create_lockbutton ());
 
   return retval;
 }
