@@ -44,25 +44,27 @@ G_BEGIN_DECLS
 #define GTK_IS_ASPECT_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_ASPECT_FRAME))
 #define GTK_ASPECT_FRAME_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_ASPECT_FRAME, GtkAspectFrameClass))
 
-
-typedef struct _GtkAspectFrame       GtkAspectFrame;
-typedef struct _GtkAspectFrameClass  GtkAspectFrameClass;
+typedef struct _GtkAspectFrame              GtkAspectFrame;
+typedef struct _GtkAspectFramePrivate       GtkAspectFramePrivate;
+typedef struct _GtkAspectFrameClass         GtkAspectFrameClass;
 
 struct _GtkAspectFrame
 {
   GtkFrame frame;
 
-  gfloat   GSEAL (xalign);
-  gfloat   GSEAL (yalign);
-  gfloat   GSEAL (ratio);
-  gboolean GSEAL (obey_child);
-
-  GtkAllocation GSEAL (center_allocation);
+  /*< private >*/
+  GtkAspectFramePrivate *priv;
 };
 
 struct _GtkAspectFrameClass
 {
   GtkFrameClass parent_class;
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 
