@@ -30,31 +30,38 @@
 #include <math.h>
 #include <stdlib.h>
 
-#include "gdk/gdkkeysyms.h"
-#include "gtkscale.h"
+#include "gtkscaleprivate.h"
 #include "gtkiconfactory.h"
 #include "gtkicontheme.h"
 #include "gtkmarshalers.h"
 #include "gtkbindings.h"
 #include "gtkorientable.h"
+#include "gtktypebuiltins.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkbuildable.h"
 #include "gtkbuilderprivate.h"
 
+#include "a11y/gtkscaleaccessible.h"
+
 
 /**
  * SECTION:gtkscale
- * @Short_description: Base class for GtkHScale and GtkVScale
+ * @Short_description: A slider widget for selecting a value from a range
  * @Title: GtkScale
  *
- * A GtkScale is a slider control used to select a numeric value.
+ * A GtkScale is a slider control used to select a numeric value. 
  * To use it, you'll probably want to investigate the methods on
  * its base class, #GtkRange, in addition to the methods for GtkScale itself.
  * To set the value of a scale, you would normally use gtk_range_set_value().
  * To detect changes to the value, you would normally use the
  * #GtkRange::value-changed signal.
  *
+ * Note that using the same upper and lower bounds for the #GtkScale (through
+ * the #GtkRange methods) will hide the slider itself. This is useful for
+ * applications that want to show an undeterminate value on the scale, without
+ * changing the layout of the application (such as movie or music players).
+ *
  * <refsect2 id="GtkScale-BUILDER-UI"><title>GtkScale as GtkBuildable</title>
  * GtkScale supports a custom &lt;marks&gt; element, which
  * can contain multiple &lt;mark&gt; elements. The "value" and "position"
@@ -72,7 +79,6 @@
 				 *    unrelated code portions otherwise
 				 */
 
-
 typedef struct _GtkScaleMark GtkScaleMark;
 
 struct _GtkScalePrivate
@@ -91,7 +97,7 @@ struct _GtkScaleMark
 {
   gdouble          value;
   gchar           *markup;
-  GtkPositionType  position;
+  GtkPositionType  position; /* always GTK_POS_TOP or GTK_POS_BOTTOM */
 };
 
 enum {
@@ -122,8 +128,7 @@ static void     gtk_scale_get_preferred_width     (GtkWidget      *widget,
 static void     gtk_scale_get_preferred_height    (GtkWidget      *widget,
                                                    gint           *minimum,
                                                    gint           *natural);
-static void     gtk_scale_style_set               (GtkWidget      *widget,
-                                                   GtkStyle       *previous);
+static void     gtk_scale_style_updated           (GtkWidget      *widget);
 static void     gtk_scale_get_range_border        (GtkRange       *range,
                                                    GtkBorder      *border);
 static void     gtk_scale_get_mark_label_size     (GtkScale        *scale,
@@ -199,7 +204,7 @@ gtk_scale_class_init (GtkScaleClass *class)
   gobject_class->get_property = gtk_scale_get_property;
   gobject_class->finalize = gtk_scale_finalize;
 
-  widget_class->style_set = gtk_scale_style_set;
+  widget_class->style_updated = gtk_scale_style_updated;
   widget_class->screen_changed = gtk_scale_screen_changed;
   widget_class->draw = gtk_scale_draw;
   widget_class->get_preferred_width = gtk_scale_get_preferred_width;
@@ -407,6 +412,8 @@ gtk_scale_class_init (GtkScaleClass *class)
                       GTK_SCROLL_END);
 
   g_type_class_add_private (gobject_class, sizeof (GtkScalePrivate));
+
+  gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_SCALE_ACCESSIBLE);
 }
 
 static void
@@ -425,6 +432,7 @@ gtk_scale_init (GtkScale *scale)
 {
   GtkScalePrivate *priv;
   GtkRange *range = GTK_RANGE (scale);
+  GtkStyleContext *context;
 
   scale->priv = G_TYPE_INSTANCE_GET_PRIVATE (scale,
                                              GTK_TYPE_SCALE,
@@ -438,12 +446,15 @@ gtk_scale_init (GtkScale *scale)
   priv->draw_value = TRUE;
   priv->value_pos = GTK_POS_TOP;
   priv->digits = 1;
-  _gtk_range_set_round_digits (range, priv->digits);
+  gtk_range_set_round_digits (range, priv->digits);
 
   gtk_scale_orientation_notify (range, NULL);
   g_signal_connect (scale, "notify::orientation",
                     G_CALLBACK (gtk_scale_orientation_notify),
                     NULL);
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE);
 }
 
 static void
@@ -502,8 +513,8 @@ gtk_scale_get_property (GObject      *object,
 /**
  * gtk_scale_new:
  * @orientation: the scale's orientation.
- * @adjustment: the #GtkAdjustment which sets the range of the scale, or
- *              %NULL to create a new adjustment.
+ * @adjustment: (allow-none): the #GtkAdjustment which sets the range
+ *              of the scale, or %NULL to create a new adjustment.
  *
  * Creates a new #GtkScale.
  *
@@ -605,7 +616,7 @@ gtk_scale_set_digits (GtkScale *scale,
     {
       priv->digits = digits;
       if (priv->draw_value)
-        _gtk_range_set_round_digits (range, digits);
+        gtk_range_set_round_digits (range, digits);
 
       _gtk_scale_clear_layout (scale);
       gtk_widget_queue_resize (GTK_WIDGET (scale));
@@ -654,9 +665,9 @@ gtk_scale_set_draw_value (GtkScale *scale,
     {
       priv->draw_value = draw_value;
       if (draw_value)
-        _gtk_range_set_round_digits (GTK_RANGE (scale), priv->digits);
+        gtk_range_set_round_digits (GTK_RANGE (scale), priv->digits);
       else
-        _gtk_range_set_round_digits (GTK_RANGE (scale), -1);
+        gtk_range_set_round_digits (GTK_RANGE (scale), -1);
 
       _gtk_scale_clear_layout (scale);
 
@@ -784,17 +795,17 @@ gtk_scale_get_range_border (GtkRange  *range,
                             NULL);
 
 
+      gtk_scale_get_mark_label_size (scale, GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
+
       if (gtk_orientable_get_orientation (GTK_ORIENTABLE (scale)) == GTK_ORIENTATION_HORIZONTAL)
         {
-          gtk_scale_get_mark_label_size (scale, GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
           if (n1 > 0)
             border->top += h1 + value_spacing + slider_width / 2;
           if (n2 > 0)
-            border->bottom += h2 + value_spacing + slider_width / 2; 
+            border->bottom += h2 + value_spacing + slider_width / 2;
         }
       else
         {
-          gtk_scale_get_mark_label_size (scale, GTK_POS_LEFT, &n1, &w1, &h1, &n2, &w2, &h2);
           if (n1 > 0)
             border->left += w1 + value_spacing + slider_width / 2;
           if (n2 > 0)
@@ -889,8 +900,8 @@ gtk_scale_get_mark_label_size (GtkScale        *scale,
           pango_layout_set_markup (layout, mark->markup, -1);
           pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
-	  w = logical_rect.width;
-	  h = logical_rect.height;
+          w = logical_rect.width;
+          h = logical_rect.height;
         }
       else
         {
@@ -916,8 +927,7 @@ gtk_scale_get_mark_label_size (GtkScale        *scale,
 }
 
 static void
-gtk_scale_style_set (GtkWidget *widget,
-                     GtkStyle  *previous)
+gtk_scale_style_updated (GtkWidget *widget)
 {
   gint slider_length;
   GtkRange *range;
@@ -932,7 +942,7 @@ gtk_scale_style_set (GtkWidget *widget,
 
   _gtk_scale_clear_layout (GTK_SCALE (widget));
 
-  GTK_WIDGET_CLASS (gtk_scale_parent_class)->style_set (widget, previous);
+  GTK_WIDGET_CLASS (gtk_scale_parent_class)->style_updated (widget);
 }
 
 static void
@@ -984,7 +994,7 @@ gtk_scale_get_preferred_height (GtkWidget *widget,
 
       gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
 
-      gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_LEFT, &n1, &w1, &h1, &n2, &w2, &h2);
+      gtk_scale_get_mark_label_size (GTK_SCALE (widget), GTK_POS_TOP, &n1, &w1, &h1, &n2, &w2, &h2);
       h1 = (n1 - 1) * h1 + MAX (h1, slider_length);
       h2 = (n2 - 1) * h1 + MAX (h2, slider_length);
       h = MAX (h1, h2);
@@ -1014,7 +1024,7 @@ find_next_pos (GtkWidget      *widget,
     }
 
   gtk_widget_get_allocation (widget, &allocation);
-  if (pos == GTK_POS_TOP || pos == GTK_POS_BOTTOM)
+  if (gtk_orientable_get_orientation (GTK_ORIENTABLE (widget)) == GTK_ORIENTATION_HORIZONTAL)
     return allocation.width;
   else
     return allocation.height;
@@ -1027,20 +1037,19 @@ gtk_scale_draw (GtkWidget *widget,
   GtkScale *scale = GTK_SCALE (widget);
   GtkScalePrivate *priv = scale->priv;
   GtkRange *range = GTK_RANGE (scale);
-  GtkStateType state_type;
-  GtkStyle *style;
-  gint n_marks;
+  GtkStateFlags state = 0;
+  GtkStyleContext *context;
   gint *marks;
   gint focus_padding;
   gint slider_width;
   gint value_spacing;
   gint min_sep = 4;
 
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
   gtk_widget_style_get (widget,
                         "focus-padding", &focus_padding,
-                        "slider-width", &slider_width, 
-                        "value-spacing", &value_spacing, 
+                        "slider-width", &slider_width,
+                        "value-spacing", &value_spacing,
                         NULL);
 
   /* We need to chain up _first_ so the various geometry members of
@@ -1048,9 +1057,8 @@ gtk_scale_draw (GtkWidget *widget,
    */
   GTK_WIDGET_CLASS (gtk_scale_parent_class)->draw (widget, cr);
 
-  state_type = GTK_STATE_NORMAL;
   if (!gtk_widget_is_sensitive (widget))
-    state_type = GTK_STATE_INSENSITIVE;
+    state |= GTK_STATE_FLAG_INSENSITIVE;
 
   if (priv->marks)
     {
@@ -1065,7 +1073,7 @@ gtk_scale_draw (GtkWidget *widget,
       gint min_pos, max_pos;
 
       orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (range));
-      n_marks = _gtk_range_get_stop_positions (range, &marks);
+      _gtk_range_get_stop_positions (range, &marks);
       layout = gtk_widget_create_pango_layout (widget, NULL);
       gtk_range_get_range_rect (range, &range_rect);
 
@@ -1090,11 +1098,14 @@ gtk_scale_draw (GtkWidget *widget,
                   y1 = range_rect.y + range_rect.height;
                   y2 = y1 + slider_width / 2;
                   min_pos = min_pos_after;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP, 0) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM, 1) - min_sep;
                 }
 
-              gtk_paint_vline (style, cr, state_type,
-                               widget, "scale-mark", y1, y2, x1);
+              gtk_style_context_save (context);
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
+              gtk_style_context_set_state (context, state);
+
+              gtk_render_line (context, cr, x1, y1, x1, y2);
 
               if (mark->markup)
                 {
@@ -1119,31 +1130,34 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = x3 + logical_rect.width + min_sep;
                     }
 
-                  gtk_paint_layout (style, cr, state_type,
-                                    FALSE, widget, "scale-mark",
-                                    x3, y3, layout);
+                  gtk_render_layout (context, cr, x3, y3, layout);
                 }
+
+              gtk_style_context_restore (context);
             }
           else
             {
-              if (mark->position == GTK_POS_LEFT)
+              if (mark->position == GTK_POS_TOP)
                 {
                   x1 = range_rect.x;
                   x2 = range_rect.x - slider_width / 2;
                   min_pos = min_pos_before;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_LEFT, 1) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP, 1) - min_sep;
                 }
               else
                 {
                   x1 = range_rect.x + range_rect.width;
                   x2 = range_rect.x + range_rect.width + slider_width / 2;
                   min_pos = min_pos_after;
-                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_LEFT, 0) - min_sep;
+                  max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM, 1) - min_sep;
                 }
               y1 = marks[i];
 
-              gtk_paint_hline (style, cr, state_type,
-                               widget, "range-mark", x1, x2, y1);
+              gtk_style_context_save (context);
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_MARK);
+              gtk_style_context_set_state (context, state);
+
+              gtk_render_line (context, cr, x1, y1, x2, y1);
 
               if (mark->markup)
                 {
@@ -1157,7 +1171,7 @@ gtk_scale_draw (GtkWidget *widget,
                     y3 = max_pos - logical_rect.height;
                   if (y3 < 0)
                     y3 = 0;
-                  if (mark->position == GTK_POS_LEFT)
+                  if (mark->position == GTK_POS_TOP)
                     {
                       x3 = x2 - value_spacing - logical_rect.width;
                       min_pos_before = y3 + logical_rect.height + min_sep;
@@ -1168,12 +1182,12 @@ gtk_scale_draw (GtkWidget *widget,
                       min_pos_after = y3 + logical_rect.height + min_sep;
                     }
 
-                  gtk_paint_layout (style, cr, state_type,
-                                    FALSE, widget, "scale-mark",
-                                    x3, y3, layout);
+                  gtk_render_layout (context, cr, x3, y3, layout);
                 }
+
+              gtk_style_context_restore (context);
             }
-        } 
+        }
 
       g_object_unref (layout);
       g_free (marks);
@@ -1181,28 +1195,19 @@ gtk_scale_draw (GtkWidget *widget,
 
   if (priv->draw_value)
     {
-      GtkOrientation orientation;
       GtkAllocation allocation;
 
       PangoLayout *layout;
       gint x, y;
 
-      orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (range));
       layout = gtk_scale_get_layout (scale);
       gtk_scale_get_layout_offsets (scale, &x, &y);
       gtk_widget_get_allocation (widget, &allocation);
 
-      gtk_paint_layout (style,
-                        cr,
-                        state_type,
-			FALSE,
-                        widget,
-                        orientation == GTK_ORIENTATION_HORIZONTAL ?
-                        "hscale" : "vscale",
-                        x - allocation.x,
-                        y - allocation.y,
-                        layout);
-
+      gtk_render_layout (context, cr,
+                         x - allocation.x,
+                         y - allocation.y,
+                         layout);
     }
 
   return FALSE;
@@ -1333,8 +1338,7 @@ _gtk_scale_format_value (GtkScale *scale,
   if (fmt)
     return fmt;
   else
-    /* insert a LRM, to prevent -20 to come out as 20- in RTL locales */
-    return g_strdup_printf ("\342\200\216%0.*f", priv->digits, value);
+    return g_strdup_printf ("%0.*f", priv->digits, value);
 }
 
 static void
@@ -1391,8 +1395,8 @@ gtk_scale_get_layout (GtkScale *scale)
 /**
  * gtk_scale_get_layout_offsets:
  * @scale: a #GtkScale
- * @x: (allow-none): location to store X offset of layout, or %NULL
- * @y: (allow-none): location to store Y offset of layout, or %NULL
+ * @x: (out) (allow-none): location to store X offset of layout, or %NULL
+ * @y: (out) (allow-none): location to store Y offset of layout, or %NULL
  *
  * Obtains the coordinates where the scale will draw the 
  * #PangoLayout representing the text in the scale. Remember
@@ -1457,6 +1461,7 @@ void
 gtk_scale_clear_marks (GtkScale *scale)
 {
   GtkScalePrivate *priv;
+  GtkStyleContext *context;
 
   g_return_if_fail (GTK_IS_SCALE (scale));
 
@@ -1466,6 +1471,10 @@ gtk_scale_clear_marks (GtkScale *scale)
   g_slist_free (priv->marks);
   priv->marks = NULL;
 
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+  gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+
   _gtk_range_set_stop_values (GTK_RANGE (scale), NULL, 0);
 
   gtk_widget_queue_resize (GTK_WIDGET (scale));
@@ -1484,22 +1493,22 @@ compare_marks (gpointer a, gpointer b)
 /**
  * gtk_scale_add_mark:
  * @scale: a #GtkScale
- * @value: the value at which the mark is placed, must be between 
+ * @value: the value at which the mark is placed, must be between
  *   the lower and upper limits of the scales' adjustment
  * @position: where to draw the mark. For a horizontal scale, #GTK_POS_TOP
- *   is drawn above the scale, anything else below. For a vertical scale,
- *   #GTK_POS_LEFT is drawn to the left of the scale, anything else to the
- *   right.
+ *   and %GTK_POS_LEFT are drawn above the scale, anything else below.
+ *   For a vertical scale, #GTK_POS_LEFT and %GTK_POS_TOP are drawn to
+ *   the left of the scale, anything else to the right.
  * @markup: (allow-none): Text to be shown at the mark, using <link linkend="PangoMarkupFormat">Pango markup</link>, or %NULL
  *
  *
- * Adds a mark at @value. 
+ * Adds a mark at @value.
  *
- * A mark is indicated visually by drawing a tick mark next to the scale, 
- * and GTK+ makes it easy for the user to position the scale exactly at the 
+ * A mark is indicated visually by drawing a tick mark next to the scale,
+ * and GTK+ makes it easy for the user to position the scale exactly at the
  * marks value.
  *
- * If @markup is not %NULL, text is shown next to the tick mark. 
+ * If @markup is not %NULL, text is shown next to the tick mark.
  *
  * To remove marks from a scale, use gtk_scale_clear_marks().
  *
@@ -1516,6 +1525,8 @@ gtk_scale_add_mark (GtkScale        *scale,
   GSList *m;
   gdouble *values;
   gint n, i;
+  GtkStyleContext *context;
+  int all_pos;
 
   g_return_if_fail (GTK_IS_SCALE (scale));
 
@@ -1524,23 +1535,49 @@ gtk_scale_add_mark (GtkScale        *scale,
   mark = g_new (GtkScaleMark, 1);
   mark->value = value;
   mark->markup = g_strdup (markup);
-  mark->position = position;
- 
+  if (position == GTK_POS_LEFT ||
+      position == GTK_POS_TOP)
+    mark->position = GTK_POS_TOP;
+  else
+    mark->position = GTK_POS_BOTTOM;
+
   priv->marks = g_slist_insert_sorted (priv->marks, mark,
                                        (GCompareFunc) compare_marks);
 
+#define MARKS_ABOVE 1
+#define MARKS_BELOW 2
+
+  all_pos = 0;
   n = g_slist_length (priv->marks);
   values = g_new (gdouble, n);
   for (m = priv->marks, i = 0; m; m = m->next, i++)
     {
       mark = m->data;
       values[i] = mark->value;
+      if (mark->position == GTK_POS_TOP)
+        all_pos |= MARKS_ABOVE;
+      else
+        all_pos |= MARKS_BELOW;
     }
-  
+
   _gtk_range_set_stop_values (GTK_RANGE (scale), values, n);
 
   g_free (values);
 
+  /* Set the style classes for the slider, so it could
+   * point to the right direction when marks are present
+   */
+  context = gtk_widget_get_style_context (GTK_WIDGET (scale));
+
+  if (all_pos & MARKS_ABOVE)
+    gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+  else
+    gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_ABOVE);
+  if (all_pos & MARKS_BELOW)
+    gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+  else
+    gtk_style_context_remove_class (context, GTK_STYLE_CLASS_SCALE_HAS_MARKS_BELOW);
+
   gtk_widget_queue_resize (GTK_WIDGET (scale));
 }
 
@@ -1665,7 +1702,11 @@ marks_start_element (GMarkupParseContext *context,
 
       mark = g_slice_new (MarkData);
       mark->value = value;
-      mark->position = position;
+      if (position == GTK_POS_LEFT ||
+          position == GTK_POS_TOP)
+        mark->position = GTK_POS_TOP;
+      else
+        mark->position = GTK_POS_BOTTOM;
       mark->markup = g_string_new ("");
       mark->context = g_strdup (msg_context);
       mark->translatable = translatable;
