@@ -64,14 +64,15 @@ typedef enum
 
 /* forward declaration to avoid excessive includes (and concurrent includes)
  */
-typedef struct _GtkRequisition	   GtkRequisition;
-typedef struct _GtkSelectionData   GtkSelectionData;
-typedef struct _GtkWidgetPrivate   GtkWidgetPrivate;
-typedef struct _GtkWidgetClass	   GtkWidgetClass;
-typedef struct _GtkWidgetAuxInfo   GtkWidgetAuxInfo;
-typedef struct _GtkClipboard	   GtkClipboard;
-typedef struct _GtkTooltip         GtkTooltip;
-typedef struct _GtkWindow          GtkWindow;
+typedef struct _GtkRequisition	       GtkRequisition;
+typedef struct _GtkSelectionData       GtkSelectionData;
+typedef struct _GtkWidgetPrivate       GtkWidgetPrivate;
+typedef struct _GtkWidgetClass	       GtkWidgetClass;
+typedef struct _GtkWidgetClassPrivate  GtkWidgetClassPrivate;
+typedef struct _GtkWidgetAuxInfo       GtkWidgetAuxInfo;
+typedef struct _GtkClipboard	       GtkClipboard;
+typedef struct _GtkTooltip             GtkTooltip;
+typedef struct _GtkWindow              GtkWindow;
 
 
 /**
@@ -139,12 +140,15 @@ struct _GtkWidget
  *   %GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT mode.
  *   %GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH means the widget prefers to have
  *   #GtkWidgetClass.get_preferred_width() called and then
- *   #GtkWidgetClass.get_preferred_height_for_width() and is the default
- *   return for unimplemented cases.
- *   However it's important to note (as described below) that any widget
- *   which trades height-for-width must respond properly to both
- *   #GtkSizeRequestModes since it might be queried in either orientation
- *   by its parent container.
+ *   #GtkWidgetClass.get_preferred_height_for_width().
+ *   %GTK_SIZE_REQUEST_CONSTANT_SIZE disables any height-for-width or
+ *   width-for-height geometry management for a said widget and is the
+ *   default return.
+ *   It's important to note (as described below) that any widget
+ *   which trades height-for-width or width-for-height must respond properly 
+ *   to both of the virtual methods #GtkWidgetClass.get_preferred_height_for_width()
+ *   and #GtkWidgetClass.get_preferred_width_for_height() since it might be 
+ *   queried in either #GtkSizeRequestMode by its parent container.
  * @get_preferred_height: This is called by containers to obtain the minimum
  *   and natural height of a widget. A widget that does not actually trade
  *   any height for width or width for height only has to implement these
@@ -425,8 +429,9 @@ struct _GtkWidgetClass
 
   /*< private >*/
 
+  GtkWidgetClassPrivate *priv;
+
   /* Padding for future expansion */
-  void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
   void (*_gtk_reserved4) (void);
@@ -477,7 +482,7 @@ void	   gtk_widget_queue_draw_area	  (GtkWidget	       *widget,
 					   gint                 width,
 					   gint                 height);
 void	   gtk_widget_queue_draw_region   (GtkWidget	       *widget,
-                                           cairo_region_t      *region);
+                                           const cairo_region_t*region);
 void	   gtk_widget_queue_resize	  (GtkWidget	       *widget);
 void	   gtk_widget_queue_resize_no_redraw (GtkWidget *widget);
 #ifndef GTK_DISABLE_DEPRECATED
@@ -523,8 +528,6 @@ gboolean   gtk_widget_remove_accelerator  (GtkWidget           *widget,
 void       gtk_widget_set_accel_path      (GtkWidget           *widget,
 					   const gchar         *accel_path,
 					   GtkAccelGroup       *accel_group);
-const gchar* _gtk_widget_get_accel_path   (GtkWidget           *widget,
-					   gboolean	       *locked);
 GList*     gtk_widget_list_accel_closures (GtkWidget	       *widget);
 gboolean   gtk_widget_can_activate_accel  (GtkWidget           *widget,
                                            guint                signal_id);
@@ -557,6 +560,7 @@ void       gtk_widget_set_can_focus       (GtkWidget           *widget,
 gboolean   gtk_widget_get_can_focus       (GtkWidget           *widget);
 gboolean   gtk_widget_has_focus           (GtkWidget           *widget);
 gboolean   gtk_widget_is_focus            (GtkWidget           *widget);
+gboolean   gtk_widget_has_visible_focus   (GtkWidget           *widget);
 void       gtk_widget_grab_focus          (GtkWidget           *widget);
 
 void       gtk_widget_set_can_default     (GtkWidget           *widget,
@@ -577,7 +581,7 @@ gboolean   gtk_widget_device_is_shadowed  (GtkWidget           *widget,
 
 void                  gtk_widget_set_name               (GtkWidget    *widget,
 							 const gchar  *name);
-G_CONST_RETURN gchar* gtk_widget_get_name               (GtkWidget    *widget);
+const gchar *         gtk_widget_get_name               (GtkWidget    *widget);
 
 void                  gtk_widget_set_state              (GtkWidget    *widget,
 							 GtkStateType  state);
@@ -719,6 +723,10 @@ void             gtk_widget_set_support_multidevice (GtkWidget      *widget,
                                                      gboolean        support_multidevice);
 
 /* Accessibility support */
+void             gtk_widget_class_set_accessible_type    (GtkWidgetClass     *widget_class,
+                                                          GType               type);
+void             gtk_widget_class_set_accessible_role    (GtkWidgetClass     *widget_class,
+                                                          AtkRole             role);
 AtkObject*       gtk_widget_get_accessible               (GtkWidget          *widget);
 
 
@@ -935,47 +943,6 @@ GtkRequisition *gtk_requisition_new      (void) G_GNUC_MALLOC;
 GtkRequisition *gtk_requisition_copy     (const GtkRequisition *requisition);
 void            gtk_requisition_free     (GtkRequisition       *requisition);
 
-GdkEventExpose *  _gtk_cairo_get_event                    (cairo_t      *cr);
-
-void              _gtk_widget_draw_internal               (GtkWidget    *widget,
-                                                           cairo_t      *cr,
-                                                           gboolean      clip_to_size);
-void              _gtk_widget_set_has_default             (GtkWidget    *widget,
-                                                           gboolean      has_default);
-void              _gtk_widget_set_has_grab                (GtkWidget    *widget,
-                                                           gboolean      has_grab);
-void              _gtk_widget_set_is_toplevel             (GtkWidget    *widget,
-                                                           gboolean      is_toplevel);
-
-void              _gtk_widget_grab_notify                 (GtkWidget    *widget,
-						           gboolean	was_grabbed);
-
-GtkWidgetAuxInfo *_gtk_widget_get_aux_info                (GtkWidget    *widget,
-							   gboolean      create);
-void              _gtk_widget_propagate_hierarchy_changed (GtkWidget    *widget,
-							   GtkWidget    *previous_toplevel);
-void              _gtk_widget_propagate_screen_changed    (GtkWidget    *widget,
-							   GdkScreen    *previous_screen);
-void		  _gtk_widget_propagate_composited_changed (GtkWidget    *widget);
-
-void	   _gtk_widget_set_device_window   (GtkWidget      *widget,
-                                            GdkDevice      *device,
-					    GdkWindow      *pointer_window);
-GdkWindow *_gtk_widget_get_device_window   (GtkWidget      *widget,
-                                            GdkDevice      *device);
-GList *    _gtk_widget_list_devices        (GtkWidget      *widget);
-
-void       _gtk_widget_synthesize_crossing (GtkWidget      *from,
-					    GtkWidget      *to,
-                                            GdkDevice      *device,
-					    GdkCrossingMode mode);
-
-gpointer     _gtk_widget_peek_request_cache (GtkWidget *widget);
-
-void         _gtk_widget_buildable_finish_accelerator (GtkWidget *widget,
-						       GtkWidget *toplevel,
-						       gpointer   user_data);
-
 gboolean     gtk_widget_in_destruction (GtkWidget *widget);
 
 GtkStyleContext * gtk_widget_get_style_context (GtkWidget *widget);
