@@ -42,7 +42,7 @@ typedef struct _CalendarData
 
   GHashTable    *details_table;
   GtkTextBuffer *details_buffer;
-  guint          details_changed;
+  gulong         details_changed;
 } CalendarData;
 
 enum
@@ -162,15 +162,13 @@ calendar_day_selected_double_click (GtkWidget    *widget,
 
   calendar_date_to_string (data, buffer+27, 256-27);
   calendar_set_signal_strings (buffer, data);
-
   gtk_calendar_get_date (GTK_CALENDAR (data->window),
-			 NULL, NULL, &day);
+                         NULL, NULL, &day);
 
-  if (GTK_CALENDAR (data->window)->marked_date[day-1] == 0) {
-    gtk_calendar_mark_day (GTK_CALENDAR (data->window), day);
-  } else { 
+  if (gtk_calendar_get_day_is_marked (GTK_CALENDAR (data->window), day))
     gtk_calendar_unmark_day (GTK_CALENDAR (data->window), day);
-  }
+  else
+    gtk_calendar_mark_day (GTK_CALENDAR (data->window), day);
 }
 
 static void
@@ -245,18 +243,15 @@ void calendar_select_font (GtkWidget    *button,
                                  CalendarData *calendar)
 {
   const char *font = NULL;
-  GtkRcStyle *style;
+  PangoFontDescription *font_desc;
 
   if (calendar->window)
-    font = gtk_font_button_get_font_name (GTK_FONT_BUTTON (button));
-
-  if (font)
-	{
-	  style = gtk_rc_style_new ();
-	  pango_font_description_free (style->font_desc);
-      style->font_desc = pango_font_description_from_string (font);
-	  gtk_widget_modify_style (calendar->window, style);
-	}
+    {
+      font = gtk_font_button_get_font_name (GTK_FONT_BUTTON (button));
+      font_desc = pango_font_description_from_string (font);
+      gtk_widget_override_font (calendar->window, font_desc);
+      pango_font_description_free (font_desc);
+    }
 }
 
 static gchar*
@@ -298,37 +293,19 @@ demonstrate_details (CalendarData *data)
 {
   static char *rainbow[] = { "#900", "#980", "#390", "#095", "#059", "#309", "#908" };
   GtkCalendar *calendar = GTK_CALENDAR (data->calendar_widget);
-  gint row, col;
+  guint year, month, day;
+  gchar *detail;
 
-  for (row = 0; row < 6; ++row)
-    for (col = 0; col < 7; ++col)
-      {
-        gint year, month, day;
-        gchar *detail;
-    
-        year = calendar->year;
-        month = calendar->month;
-        month += calendar->day_month[row][col];
-        day = calendar->day[row][col];
-    
-        if (month < 1)
-          {
-            month += 12;
-            year -= 1;
-          }
-        else if (month > 12)
-          {
-            month -= 12;
-            year += 1;
-  }
+  gtk_calendar_get_date (calendar,
+                         &year, &month, &day);
 
-        detail = g_strdup_printf ("<span color='%s'>yadda\n"
-                                  "(%04d-%02d-%02d)</span>",
-                                  rainbow[(day - 1) % 7],
-                                  year, month, day);
-
-        calendar_set_detail (data, year, month - 1, day, detail);
-      }
+  for (day = 0; day < 29; ++day)
+    {
+      detail = g_strdup_printf ("<span color='%s'>yadda\n"
+                                "(%04d-%02d-%02d)</span>",
+                                rainbow[(day - 1) % 7], year, month, day);
+      calendar_set_detail (data, year, month, day, detail);
+   }
 
   gtk_widget_queue_resize (data->calendar_widget);
   calendar_update_details (data);
@@ -419,7 +396,8 @@ create_calendar(void)
   GtkWidget *frame, *label, *bbox, *align, *details;
 
   GtkSizeGroup *size;
-  GtkStyle *style;
+  GtkStyleContext *context;
+  const PangoFontDescription *font_desc;
   gchar *font;
   gint i;
   
@@ -453,7 +431,7 @@ create_calendar(void)
 		    G_CALLBACK (gtk_false),
 		    NULL);
 
-  hpaned = gtk_hpaned_new ();
+  hpaned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 
   /* Calendar widget */
 
@@ -488,19 +466,19 @@ create_calendar(void)
 		    G_CALLBACK (calendar_next_year),
 		    &calendar_data);
 
-  rpane = gtk_vbox_new (FALSE, DEF_PAD_SMALL);
+  rpane = gtk_box_new (GTK_ORIENTATION_VERTICAL, DEF_PAD_SMALL);
   gtk_paned_pack2 (GTK_PANED (hpaned), rpane, FALSE, FALSE);
 
   /* Build the right font-button */
 
-  vbox = gtk_vbox_new(FALSE, DEF_PAD_SMALL);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, DEF_PAD_SMALL);
   frame = create_frame ("<b>Options</b>", vbox, 1, 0);
   gtk_box_pack_start (GTK_BOX (rpane), frame, FALSE, TRUE, 0);
   size = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-  gtk_widget_ensure_style (calendar);
-  style = gtk_widget_get_style (calendar);
-  font = pango_font_description_to_string (style->font_desc);
+  context = gtk_widget_get_style_context (calendar);
+  font_desc = gtk_style_context_get_font (context, GTK_STATE_FLAG_NORMAL);
+  font = pango_font_description_to_string (font_desc);
   button = gtk_font_button_new_with_font (font);
   g_free (font);
 
@@ -510,10 +488,11 @@ create_calendar(void)
 
   label = gtk_label_new_with_mnemonic ("_Font:");
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
   gtk_size_group_add_widget (size, label);
 
-  hbox = gtk_hbox_new (FALSE, DEF_PAD_SMALL);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, DEF_PAD_SMALL);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
@@ -530,10 +509,11 @@ create_calendar(void)
 
   label = gtk_label_new_with_mnemonic ("Details W_idth:");
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
   gtk_size_group_add_widget (size, label);
 
-  hbox = gtk_hbox_new (FALSE, DEF_PAD_SMALL);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, DEF_PAD_SMALL);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
@@ -550,17 +530,18 @@ create_calendar(void)
 
   label = gtk_label_new_with_mnemonic ("Details H_eight:");
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
   gtk_size_group_add_widget (size, label);
 
-  hbox = gtk_hbox_new (FALSE, DEF_PAD_SMALL);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, DEF_PAD_SMALL);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
 
   /* Build the right details frame */
 
-  vbox = gtk_vbox_new(FALSE, DEF_PAD_SMALL);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, DEF_PAD_SMALL);
   frame = create_frame ("<b>Details</b>", vbox, 1, 1);
   gtk_box_pack_start (GTK_BOX (rpane), frame, FALSE, TRUE, 0);
 
@@ -582,7 +563,7 @@ create_calendar(void)
 
   gtk_box_pack_start (GTK_BOX (vbox), scroller, FALSE, TRUE, 0);
 
-  hbox = gtk_hbox_new (FALSE, DEF_PAD_SMALL);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, DEF_PAD_SMALL);
   align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
   gtk_container_add (GTK_CONTAINER (align), hbox);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, TRUE, 0);
@@ -613,7 +594,7 @@ create_calendar(void)
   
   /* Build the Right frame with the flags in */ 
 
-  vbox = gtk_vbox_new(FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   frame = create_expander ("<b>Flags</b>", vbox, 1, 0);
   gtk_box_pack_start (GTK_BOX (rpane), frame, TRUE, TRUE, 0);
 
@@ -634,24 +615,25 @@ create_calendar(void)
    *  Build the Signal-event part.
    */
 
-  vbox = gtk_vbox_new (TRUE, DEF_PAD_SMALL);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, DEF_PAD_SMALL);
+  gtk_box_set_homogeneous (GTK_BOX (vbox), TRUE);
   frame = create_frame ("<b>Signal Events</b>", vbox, 1, 0);
   
-  hbox = gtk_hbox_new (FALSE, 3);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   label = gtk_label_new ("Signal:");
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   calendar_data.last_sig = gtk_label_new ("");
   gtk_box_pack_start (GTK_BOX (hbox), calendar_data.last_sig, FALSE, TRUE, 0);
 
-  hbox = gtk_hbox_new (FALSE, 3);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   label = gtk_label_new ("Previous signal:");
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   calendar_data.prev_sig = gtk_label_new ("");
   gtk_box_pack_start (GTK_BOX (hbox), calendar_data.prev_sig, FALSE, TRUE, 0);
 
-  hbox = gtk_hbox_new (FALSE, 3);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   label = gtk_label_new ("Second previous signal:");
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
@@ -662,24 +644,29 @@ create_calendar(void)
    *  Glue everything together
    */
 
-  bbox = gtk_hbutton_box_new ();
+  bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
 
   button = gtk_button_new_with_label ("Close");
   g_signal_connect (button, "clicked", G_CALLBACK (gtk_main_quit), NULL);
   gtk_container_add (GTK_CONTAINER (bbox), button);
 
-  vbox = gtk_vbox_new (FALSE, DEF_PAD_SMALL);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, DEF_PAD_SMALL);
 
-  gtk_box_pack_start (GTK_BOX (vbox), hpaned,                TRUE,  TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new (), FALSE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), frame,                 FALSE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), gtk_hseparator_new (), FALSE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), bbox,                  FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hpaned,
+                      TRUE,  TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
+                      FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), frame,
+                      FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), gtk_separator_new (GTK_ORIENTATION_HORIZONTAL),
+                      FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), bbox,
+                      FALSE, TRUE, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_set_can_default (button, TRUE);
   gtk_widget_grab_default (button);
 
   gtk_window_set_default_size (GTK_WINDOW (window), 600, 0);
@@ -692,6 +679,9 @@ int main(int   argc,
 {
   gtk_init (&argc, &argv);
 
+  if (g_getenv ("GTK_RTL"))
+    gtk_widget_set_default_direction (GTK_TEXT_DIR_RTL);
+
   create_calendar();
 
   gtk_main();
